@@ -21,6 +21,9 @@
 #include "container.h"
 #include "utils.h"
 
+#include <stdint.h>
+#include <math.h>
+
 enum
 {
   CGROUP_MEMORY = 1 << 0,
@@ -38,8 +41,13 @@ struct libcrun_cgroup_status
   char *scope;
 
   int manager;
+
+  bool bpf_dev_set;
 };
 
+/* Forward declaration for function pointers below.  */
+struct libcrun_cgroup_args;
+
 struct libcrun_cgroup_manager
 {
   /* Create a new cgroup and fill PATH in OUT.  */
@@ -52,8 +60,8 @@ struct libcrun_cgroup_manager
 };
 
 int move_process_to_cgroup (pid_t pid, const char *subsystem, const char *path, libcrun_error_t *err);
-int enter_cgroup_subsystem (pid_t pid, const char *subsystem, const char *path, bool create_if_missing,
-                            libcrun_error_t *err);
+int enter_cgroup_v1_subsystem (pid_t pid, const char *subsystem, const char *path, bool create_if_missing,
+                               libcrun_error_t *err);
 int enable_controllers (const char *path, libcrun_error_t *err);
 int chown_cgroups (const char *path, uid_t uid, gid_t gid, libcrun_error_t *err);
 int cgroup_killall_path (const char *path, int signal, libcrun_error_t *err);
@@ -76,8 +84,22 @@ int libcrun_cgroup_pause_unpause_path (const char *cgroup_path, const bool pause
 static inline uint64_t
 convert_shares_to_weight (uint64_t shares)
 {
-  /* convert linearly from 2-262144 to 1-10000.  */
-  return (1 + ((shares - 2) * 9999) / 262142);
+  double l, exponent;
+
+  /* The value of 0 means "unset".  */
+  if (shares == 0)
+    return 0;
+  if (shares <= 2)
+    return 1;
+  if (shares >= 262144)
+    return 10000;
+
+  l = log2 ((double) shares);
+
+  /* Quadratic function which fits min, max, and default.  */
+  exponent = (l * l + 125 * l) / 612.0 - 7.0 / 34.0;
+
+  return (uint64_t) ceil (pow (10, exponent));
 }
 
 int initialize_cpuset_subsystem (const char *path, libcrun_error_t *err);
