@@ -43,6 +43,8 @@
 #  define CRIU_CHECKPOINT_LOG_FILE "dump.log"
 #  define CRIU_RESTORE_LOG_FILE "restore.log"
 #  define DESCRIPTORS_FILENAME "descriptors.json"
+#  define CRIU_RUNC_CONFIG_FILE "/etc/criu/runc.conf"
+#  define CRIU_CRUN_CONFIG_FILE "/etc/criu/crun.conf"
 
 #  define CRIU_EXT_NETNS "extRootNetNS"
 #  define CRIU_EXT_PIDNS "extRootPidNS"
@@ -51,6 +53,9 @@
 #    define CLONE_NEWTIME 0x00000080 /* New time namespace */
 #  endif
 
+/* Defined in chroot_realpath.c  */
+char *chroot_realpath (const char *chroot, const char *path, char resolved_path[]);
+
 static const char *console_socket = NULL;
 
 #  define LIBCRIU_MIN_VERSION 31500
@@ -91,10 +96,12 @@ struct libcriu_wrapper_s
   int (*criu_add_cg_root) (const char *ctrl, const char *path);
   void (*criu_set_shell_job) (bool shell_job);
   void (*criu_set_tcp_established) (bool tcp_established);
+  void (*criu_set_tcp_close) (bool tcp_close);
   void (*criu_set_track_mem) (bool track_mem);
   void (*criu_set_work_dir_fd) (int fd);
   int (*criu_set_lsm_profile) (const char *name);
   int (*criu_set_lsm_mount_context) (const char *name);
+  int (*criu_set_config_file) (const char *path);
 };
 
 static struct libcriu_wrapper_s *libcriu_wrapper;
@@ -142,7 +149,7 @@ load_wrapper (struct libcriu_wrapper_s **wrapper_out, libcrun_error_t *err)
 #  ifndef STATIC
   wrapper->handle = dlopen ("libcriu.so.2", RTLD_NOW);
   if (wrapper->handle == NULL)
-    return crun_make_error (err, 0, "could not load `libcriu.so.2`");
+    return crun_make_error (err, 0, "could not load `libcriu.so.2`: `%s`", dlerror ());
 #  endif
 
   LOAD_CRIU_FUNCTION (criu_add_ext_mount, false);
@@ -185,10 +192,14 @@ load_wrapper (struct libcriu_wrapper_s **wrapper_out, libcrun_error_t *err)
   LOAD_CRIU_FUNCTION (criu_add_cg_root, false);
   LOAD_CRIU_FUNCTION (criu_set_shell_job, false);
   LOAD_CRIU_FUNCTION (criu_set_tcp_established, false);
+  LOAD_CRIU_FUNCTION (criu_set_tcp_close, false);
   LOAD_CRIU_FUNCTION (criu_set_track_mem, false);
   LOAD_CRIU_FUNCTION (criu_set_work_dir_fd, false);
   LOAD_CRIU_FUNCTION (criu_set_lsm_profile, false);
   LOAD_CRIU_FUNCTION (criu_set_lsm_mount_context, false);
+#  if ! defined STATIC || defined CRIU_CONFIG_FILE
+  LOAD_CRIU_FUNCTION (criu_set_config_file, true);
+#  endif
 
   libcriu_wrapper = *wrapper_out = wrapper;
   wrapper = NULL;
@@ -262,6 +273,54 @@ criu_check_mem_track (char *work_path, libcrun_error_t *err)
 
 #  endif
 
+static int
+register_masked_paths_mounts (runtime_spec_schema_config_schema *def, libcrun_container_t *container,
+                              struct libcriu_wrapper_s *libcriu_wrapper, bool is_restore, libcrun_error_t *err)
+{
+  cleanup_free char *empty_dir_path = NULL;
+  bool shared_dir_registered = false;
+  size_t i;
+  int ret;
+
+  for (i = 0; i < def->linux->masked_paths_len; i++)
+    {
+      struct stat statbuf;
+      ret = stat (def->linux->masked_paths[i], &statbuf);
+      if (ret != 0)
+        continue;
+
+      if (S_ISDIR (statbuf.st_mode))
+        {
+          if (! shared_dir_registered)
+            {
+              ret = get_shared_empty_directory_path (&empty_dir_path,
+                                                     (container->context ? container->context->state_root : NULL), err);
+              if (UNLIKELY (ret < 0))
+                return ret;
+
+              ret = libcriu_wrapper->criu_add_ext_mount (empty_dir_path, empty_dir_path);
+              if (UNLIKELY (ret < 0))
+                return crun_make_error (err, -ret, "CRIU: failed adding external mount for shared empty directory `%s`", empty_dir_path);
+
+              shared_dir_registered = true;
+            }
+
+          ret = libcriu_wrapper->criu_add_ext_mount (def->linux->masked_paths[i], empty_dir_path);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, -ret, "CRIU: failed adding external mount for masked directory `%s`", def->linux->masked_paths[i]);
+        }
+      else if (S_ISREG (statbuf.st_mode))
+        {
+          const char *bind_target = is_restore ? "/dev/null" : def->linux->masked_paths[i];
+          ret = libcriu_wrapper->criu_add_ext_mount (def->linux->masked_paths[i], bind_target);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, -ret, "CRIU: failed adding external mount to `%s`", bind_target);
+        }
+    }
+
+  return 0;
+}
+
 static int
 restore_cgroup_v1_mount (runtime_spec_schema_config_schema *def, libcrun_error_t *err)
 {
@@ -283,7 +342,8 @@ restore_cgroup_v1_mount (runtime_spec_schema_config_schema *def, libcrun_error_t
   /* First check if there is actually a cgroup mount in the container. */
   for (i = 0; i < def->mounts_len; i++)
     {
-      if (strcmp (def->mounts[i]->type, "cgroup") == 0)
+      char *type = def->mounts[i]->type;
+      if (type && strcmp (type, "cgroup") == 0)
         {
           has_cgroup_mount = true;
           break;
@@ -353,7 +413,8 @@ checkpoint_cgroup_v1_mount (runtime_spec_schema_config_schema *def, libcrun_erro
   /* First check if there is actually a cgroup mount in the container. */
   for (i = 0; i < def->mounts_len; i++)
     {
-      if (strcmp (def->mounts[i]->type, "cgroup") == 0)
+      char *type = def->mounts[i]->type;
+      if (type && strcmp (type, "cgroup") == 0)
         {
           has_cgroup_mount = true;
           break;
@@ -405,6 +466,38 @@ checkpoint_cgroup_v1_mount (runtime_spec_schema_config_schema *def, libcrun_erro
   return 0;
 }
 
+static int
+handle_criu_config_file (libcrun_container_t *container, libcrun_error_t *err)
+{
+  int ret;
+  const char *criu_config_annotation;
+  const char *config_file = CRIU_RUNC_CONFIG_FILE;
+
+  criu_config_annotation = find_annotation (container, "org.criu.config");
+
+  /* Ignore missing criu_set_config_file() API for compatibility with older CRIU versions,
+   * and show an error only if config file is explicitly set with annotation.
+   */
+  if (libcriu_wrapper->criu_set_config_file == NULL)
+    {
+      if (criu_config_annotation)
+        return crun_make_error (err, 0, "libcriu RPC config files supported in CRIU >= 4.2");
+
+      return 0;
+    }
+
+  if (criu_config_annotation)
+    config_file = criu_config_annotation;
+  else if (access (CRIU_CRUN_CONFIG_FILE, F_OK) == 0)
+    config_file = CRIU_CRUN_CONFIG_FILE;
+
+  ret = libcriu_wrapper->criu_set_config_file (config_file);
+  if (UNLIKELY (ret < 0))
+    return crun_make_error (err, 0, "failed to set CRIU config file");
+
+  return 0;
+}
+
 int
 libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, libcrun_container_t *container,
                                          libcrun_checkpoint_restore_t *cr_options, libcrun_error_t *err)
@@ -467,6 +560,11 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
   /* Set up logging. */
   libcriu_wrapper->criu_set_log_level (4);
   libcriu_wrapper->criu_set_log_file (CRIU_CHECKPOINT_LOG_FILE);
+
+  /* Set up CRIU config file */
+  if (UNLIKELY (handle_criu_config_file (container, err)))
+    return -1;
+
   /* Setting the pid early as we can skip a lot of checkpoint setup if
    * we just do a pre-dump. The PID needs to be set always. Do it here.
    * The main process of the container is the process CRIU will checkpoint
@@ -478,6 +576,10 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
    * crun to set it if the user has not selected a specific directory. */
   if (cr_options->work_path != NULL)
     {
+      ret = mkdir (cr_options->work_path, 0700);
+      if (UNLIKELY ((ret == -1) && (errno != EEXIST)))
+        return crun_make_error (err, errno, "error creating CRIU work directory `%s`", cr_options->work_path);
+
       work_fd = open (cr_options->work_path, O_DIRECTORY | O_CLOEXEC);
       if (UNLIKELY (work_fd == -1))
         return crun_make_error (err, errno, "error opening CRIU work directory `%s`", cr_options->work_path);
@@ -504,9 +606,19 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
         if (UNLIKELY (criu_can_mem_track == -1))
           return -1;
         libcriu_wrapper->criu_set_track_mem (true);
-        /* The parent path needs to be a relative path. CRIU will fail
-         * if the path is not in the right format. Usually something like
-         * ../previous-dump */
+
+        /* The parent path must be relative to image path (something like ../previous-dump).
+           CRIU will fail with an unclear error message if the path is not right.
+         */
+        if (UNLIKELY (cr_options->parent_path[0] == '/'))
+          return crun_make_error (err, 0, "--parent-path must be relative");
+        int is_dir = crun_dir_p_at (image_fd, cr_options->parent_path, false, err);
+        if (UNLIKELY (is_dir <= 0))
+          {
+            if (is_dir < 0)
+              return crun_error_wrap (err, "invalid --parent-path");
+            return crun_make_error (err, ENOTDIR, "invalid --parent-path");
+          }
         ret = libcriu_wrapper->criu_set_parent_images (cr_options->parent_path);
         if (UNLIKELY (ret != 0))
           return crun_make_error (err, -ret, "error setting CRIU parent images path to `%s`", cr_options->parent_path);
@@ -557,38 +669,43 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
   if (cgroup_mode != CGROUP_MODE_UNIFIED)
     {
       ret = checkpoint_cgroup_v1_mount (def, err);
-      if (UNLIKELY (ret != 0))
-        return crun_make_error (err, 0, "error handling cgroup v1 mounts");
+      if (UNLIKELY (ret < 0))
+        return ret;
     }
 
   /* Tell CRIU about external bind mounts. */
   for (i = 0; i < def->mounts_len; i++)
     {
-      size_t j;
-
-      for (j = 0; j < def->mounts[i]->options_len; j++)
+      bool nofollow = false;
+      if (is_bind_mount (def->mounts[i], NULL, &nofollow))
         {
-          if (strcmp (def->mounts[i]->options[j], "bind") == 0 || strcmp (def->mounts[i]->options[j], "rbind") == 0)
+          /* We need to resolve mount destination inside container's root for CRIU to handle. */
+          char buf[PATH_MAX];
+          const char *dest_in_root;
+
+          if (nofollow)
+            return crun_make_error (err, 0, "CRIU does not support `src-nofollow` for bind mounts");
+
+          dest_in_root = chroot_realpath (status->rootfs, def->mounts[i]->destination, buf);
+          if (UNLIKELY (dest_in_root == NULL))
             {
-              ret = libcriu_wrapper->criu_add_ext_mount (def->mounts[i]->destination, def->mounts[i]->destination);
-              if (UNLIKELY (ret < 0))
-                return crun_make_error (err, -ret, "CRIU: failed adding external mount to `%s`", def->mounts[i]->destination);
-              break;
+              if (errno != ENOENT)
+                return crun_make_error (err, errno, "unable to resolve external bind mount `%s` under rootfs", def->mounts[i]->destination);
+              else
+                dest_in_root = def->mounts[i]->destination;
             }
+          else
+            dest_in_root += strlen (status->rootfs);
+
+          ret = libcriu_wrapper->criu_add_ext_mount (dest_in_root, dest_in_root);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, -ret, "CRIU: failed adding external mount to `%s`", def->mounts[i]->destination);
         }
     }
 
-  for (i = 0; i < def->linux->masked_paths_len; i++)
-    {
-      struct stat statbuf;
-      ret = stat (def->linux->masked_paths[i], &statbuf);
-      if (ret == 0 && S_ISREG (statbuf.st_mode))
-        {
-          ret = libcriu_wrapper->criu_add_ext_mount (def->linux->masked_paths[i], def->linux->masked_paths[i]);
-          if (UNLIKELY (ret < 0))
-            return crun_make_error (err, -ret, "CRIU: failed adding external mount to `%s`", def->linux->masked_paths[i]);
-        }
-    }
+  ret = register_masked_paths_mounts (def, container, libcriu_wrapper, false, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
 
   /* CRIU tries to checkpoint and restore all namespaces. However,
    * namespaces could be shared between containers in a pod.
@@ -704,22 +821,24 @@ prepare_restore_mounts (runtime_spec_schema_config_schema *def, char *root, libc
       char *dest = def->mounts[i]->destination;
       char *type = def->mounts[i]->type;
       cleanup_close int root_fd = -1;
+      bool nofollow = false;
       bool on_tmpfs = false;
       int is_dir = 1;
       size_t j;
 
       /* cgroup restore should be handled by CRIU itself */
-      if (strcmp (type, "cgroup") == 0 || strcmp (type, "cgroup2") == 0)
+      if (type && (strcmp (type, "cgroup") == 0 || strcmp (type, "cgroup2") == 0))
         continue;
 
       /* Check if the mountpoint is on a tmpfs. CRIU restores
        * all tmpfs. We do need to recreate directories on a tmpfs. */
+      size_t dest_len = strlen (dest);
       for (j = 0; j < def->mounts_len; j++)
         {
-          cleanup_free char *dest_loop = NULL;
-
-          xasprintf (&dest_loop, "%s/", def->mounts[j]->destination);
-          if (strncmp (dest, dest_loop, strlen (dest_loop)) == 0 && strcmp (def->mounts[j]->type, "tmpfs") == 0)
+          if (def->mounts[j]->type == NULL || strcmp (def->mounts[j]->type, "tmpfs") != 0)
+            continue;
+          size_t mount_len = strlen (def->mounts[j]->destination);
+          if (mount_len < dest_len && dest[mount_len] == '/' && strncmp (dest, def->mounts[j]->destination, mount_len) == 0)
             {
               /* This is a mountpoint which is on a tmpfs.*/
               on_tmpfs = true;
@@ -731,16 +850,14 @@ prepare_restore_mounts (runtime_spec_schema_config_schema *def, char *root, libc
         continue;
 
       /* For bind mounts check if the source is a file or a directory. */
-      for (j = 0; j < def->mounts[i]->options_len; j++)
+      if (is_bind_mount (def->mounts[i], NULL, &nofollow))
         {
-          const char *opt = def->mounts[i]->options[j];
-          if (strcmp (opt, "bind") == 0 || strcmp (opt, "rbind") == 0)
-            {
-              is_dir = crun_dir_p (def->mounts[i]->source, false, err);
-              if (UNLIKELY (is_dir < 0))
-                return is_dir;
-              break;
-            }
+          if (nofollow)
+            return crun_make_error (err, 0, "CRIU does not support `src-nofollow` for bind mounts");
+
+          is_dir = crun_dir_p (def->mounts[i]->source, false, err);
+          if (UNLIKELY (is_dir < 0))
+            return is_dir;
         }
 
       root_fd = open (root, O_RDONLY | O_CLOEXEC);
@@ -857,6 +974,10 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
    * crun to set it if the user has not selected a specific directory. */
   if (cr_options->work_path != NULL)
     {
+      ret = mkdir (cr_options->work_path, 0700);
+      if (UNLIKELY ((ret == -1) && (errno != EEXIST)))
+        return crun_make_error (err, errno, "error creating CRIU work directory `%s`", cr_options->work_path);
+
       work_fd = open (cr_options->work_path, O_DIRECTORY | O_CLOEXEC);
       if (UNLIKELY (work_fd == -1))
         return crun_make_error (err, errno, "error opening CRIU work directory `%s`", cr_options->work_path);
@@ -886,31 +1007,35 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
   /* Tell CRIU about external bind mounts. */
   for (i = 0; i < def->mounts_len; i++)
     {
-      size_t j;
-
-      for (j = 0; j < def->mounts[i]->options_len; j++)
+      bool nofollow = false;
+      if (is_bind_mount (def->mounts[i], NULL, &nofollow))
         {
-          if (strcmp (def->mounts[i]->options[j], "bind") == 0 || strcmp (def->mounts[i]->options[j], "rbind") == 0)
+          /* We need to resolve mount destination inside container's root for CRIU to handle. */
+          char buf[PATH_MAX];
+          const char *dest_in_root;
+
+          if (nofollow)
+            return crun_make_error (err, 0, "CRIU does not support `src-nofollow` for bind mounts");
+
+          dest_in_root = chroot_realpath (status->rootfs, def->mounts[i]->destination, buf);
+          if (UNLIKELY (dest_in_root == NULL))
             {
-              ret = libcriu_wrapper->criu_add_ext_mount (def->mounts[i]->destination, def->mounts[i]->source);
-              if (UNLIKELY (ret < 0))
-                return crun_make_error (err, -ret, "CRIU: failed adding external mount to `%s`", def->mounts[i]->source);
-              break;
+              if (errno != ENOENT)
+                return crun_make_error (err, errno, "unable to resolve external bind mount destination `%s` under rootfs", def->mounts[i]->destination);
+              dest_in_root = def->mounts[i]->destination;
             }
+          else
+            dest_in_root += strlen (status->rootfs);
+
+          ret = libcriu_wrapper->criu_add_ext_mount (dest_in_root, def->mounts[i]->source);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, -ret, "CRIU: failed adding external mount to `%s`", def->mounts[i]->source);
         }
     }
 
-  for (i = 0; i < def->linux->masked_paths_len; i++)
-    {
-      struct stat statbuf;
-      ret = stat (def->linux->masked_paths[i], &statbuf);
-      if (ret == 0 && S_ISREG (statbuf.st_mode))
-        {
-          ret = libcriu_wrapper->criu_add_ext_mount (def->linux->masked_paths[i], "/dev/null");
-          if (UNLIKELY (ret < 0))
-            return crun_make_error (err, -ret, "CRIU: failed adding external mount to `%s`", "/dev/null");
-        }
-    }
+  ret = register_masked_paths_mounts (def, container, libcriu_wrapper, true, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
 
   /* do realpath on root */
   bundle_cleanup = realpath (status->bundle, NULL);
@@ -1019,6 +1144,10 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
 #  endif
     }
 
+  /* Set up CRIU config file */
+  if (UNLIKELY (handle_criu_config_file (container, err)))
+    return -1;
+
   /* Tell CRIU if cgroup v1 needs to be handled. */
   ret = restore_cgroup_v1_mount (def, err);
   if (UNLIKELY (ret < 0))
@@ -1031,6 +1160,7 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
   libcriu_wrapper->criu_set_ext_unix_sk (cr_options->ext_unix_sk);
   libcriu_wrapper->criu_set_shell_job (cr_options->shell_job);
   libcriu_wrapper->criu_set_tcp_established (cr_options->tcp_established);
+  libcriu_wrapper->criu_set_tcp_close (cr_options->tcp_close);
   libcriu_wrapper->criu_set_file_locks (cr_options->file_locks);
   libcriu_wrapper->criu_set_orphan_pts_master (true);
 
