@@ -200,12 +200,12 @@ check_idmapped_mounts (const char *path)
       _exit (EXIT_SUCCESS);
     }
 
-  sprintf (proc_path, "/proc/%d/uid_map", pid);
+  snprintf (proc_path, sizeof (proc_path), "/proc/%d/uid_map", pid);
   write_to (proc_path, "0 0 1");
-  sprintf (proc_path, "/proc/%d/gid_map", pid);
+  snprintf (proc_path, sizeof (proc_path), "/proc/%d/gid_map", pid);
   write_to (proc_path, "0 0 1");
 
-  sprintf (proc_path, "/proc/%d/ns/user", pid);
+  snprintf (proc_path, sizeof (proc_path), "/proc/%d/ns/user", pid);
   fd = open (proc_path, O_RDONLY);
   if (fd < 0)
     error (EXIT_FAILURE, errno, "open `%s`", proc_path);
@@ -484,6 +484,14 @@ main (int argc, char **argv)
       exit (0);
     }
 
+  if (strcmp (argv[1], "exit") == 0)
+    {
+      int code = 0;
+      if (argc >= 3)
+        code = atoi (argv[2]);
+      exit (code);
+    }
+
   if (strcmp (argv[1], "echo") == 0)
     {
       if (argc < 3)
@@ -766,12 +774,12 @@ main (int argc, char **argv)
       if (argc < 3)
         error (EXIT_FAILURE, 0, "'create-sub-cgroup-and-wait' requires an argument");
 
-      sprintf (path, "/sys/fs/cgroup/%s", argv[2]);
+      snprintf (path, sizeof (path), "/sys/fs/cgroup/%s", argv[2]);
       ret = mkdir (path, 0700);
       if (ret < 0)
         error (EXIT_FAILURE, errno, "mkdir");
 
-      sprintf (path, "/sys/fs/cgroup/%s/cgroup.procs", argv[2]);
+      snprintf (path, sizeof (path), "/sys/fs/cgroup/%s/cgroup.procs", argv[2]);
 
       fd = open (path, O_WRONLY);
       if (fd < 0)
