@@ -28,13 +28,13 @@ def test_no_caps():
     conf['process']['capabilities'] = {}
     for i in ['bounding', 'effective', 'inheritable', 'permitted', 'ambient']:
         conf['process']['capabilities'][i] = []
-    out, _ = run_and_get_output(conf)
+    out, _ = run_and_get_output(conf, hide_stderr=True)
     proc_status = parse_proc_status(out)
 
     for i in ['CapInh', 'CapPrm', 'CapEff', 'CapBnd', 'CapAmb']:
         if proc_status.get(i, '') != "0000000000000000":
             actual = proc_status.get(i, 'MISSING')
-            sys.stderr.write("# %s capability check failed: expected '0000000000000000', got '%s'\n" % (i, actual))
+            logger.info("%s capability check failed: expected '0000000000000000', got '%s'", i, actual)
             return -1
     return 0
 
@@ -45,13 +45,13 @@ def test_some_caps():
     conf['process']['capabilities'] = {}
     for i in ['bounding', 'effective', 'inheritable', 'permitted', 'ambient']:
         conf['process']['capabilities'][i] = []
-    out, _ = run_and_get_output(conf)
+    out, _ = run_and_get_output(conf, hide_stderr=True)
     proc_status = parse_proc_status(out)
 
     for i in ['CapInh', 'CapPrm', 'CapEff', 'CapBnd', 'CapAmb']:
         if proc_status.get(i, '') != "0000000000000000":
             actual = proc_status.get(i, 'MISSING')
-            sys.stderr.write("# %s capability check failed: expected '0000000000000000', got '%s'\n" % (i, actual))
+            logger.info("%s capability check failed: expected '0000000000000000', got '%s'", i, actual)
             return -1
     return 0
 
@@ -63,13 +63,13 @@ def test_unknown_caps():
     # unknown caps must be ignored
     for i in ['bounding', 'effective', 'inheritable', 'permitted', 'ambient']:
         conf['process']['capabilities'][i] = ['CAP_UNKNOWN', 'UNKNOWN_CAP']
-    out, _ = run_and_get_output(conf)
+    out, _ = run_and_get_output(conf, hide_stderr=True)
     proc_status = parse_proc_status(out)
 
     for i in ['CapInh', 'CapPrm', 'CapEff', 'CapBnd', 'CapAmb']:
         if proc_status.get(i, '') != "0000000000000000":
             actual = proc_status.get(i, 'MISSING')
-            sys.stderr.write("# %s capability check failed (unknown caps should be ignored): expected '0000000000000000', got '%s'\n" % (i, actual))
+            logger.info("%s capability check failed (unknown caps should be ignored): expected '0000000000000000', got '%s'", i, actual)
             return -1
     return 0
 
@@ -79,28 +79,27 @@ def test_new_privs():
     add_all_namespaces(conf)
 
     conf['process']['noNewPrivileges'] = True
-    out, _ = run_and_get_output(conf)
+    out, _ = run_and_get_output(conf, hide_stderr=True)
     proc_status = parse_proc_status(out)
     no_new_privs = proc_status.get('NoNewPrivs', 'MISSING')
     if no_new_privs != "1":
-        sys.stderr.write("# noNewPrivileges=true test failed: expected '1', got '%s'\n" % no_new_privs)
+        logger.info("noNewPrivileges=true test failed: expected '1', got '%s'", no_new_privs)
         return -1
 
     with open("/proc/self/status") as f:
-        host_proc_status = parse_proc_status("\n".join(f.readlines()))
+        host_proc_status = parse_proc_status(f.read())
         host_no_new_privs = host_proc_status.get('NoNewPrivs', '0')
         # if nonewprivs is already set, it cannot be unset, so skip the
         # next test
         if host_no_new_privs == "1":
-            sys.stderr.write("# skipping noNewPrivileges=false test: host already has NoNewPrivs=1\n")
-            return 0
+            return (77, "host already has NoNewPrivs=1")
 
     conf['process']['noNewPrivileges'] = False
-    out, _ = run_and_get_output(conf)
+    out, _ = run_and_get_output(conf, hide_stderr=True)
     proc_status = parse_proc_status(out)
     no_new_privs = proc_status.get('NoNewPrivs', 'MISSING')
     if no_new_privs != "0":
-        sys.stderr.write("# noNewPrivileges=false test failed: expected '0', got '%s'\n" % no_new_privs)
+        logger.info("noNewPrivileges=false test failed: expected '0', got '%s'", no_new_privs)
         return -1
 
     return 0
@@ -113,14 +112,14 @@ def helper_test_some_caps(uid, captypes, proc_name):
     conf['process']['capabilities'] = {}
     for i in captypes + ['bounding']:
         conf['process']['capabilities'][i] = ["CAP_SYS_ADMIN"]
-    out, _ = run_and_get_output(conf)
+    out, _ = run_and_get_output(conf, hide_stderr=True)
     proc_status = parse_proc_status(out)
 
     expected = "0000000000200000"
     actual = proc_status.get(proc_name, 'MISSING')
     if actual != expected:
-        sys.stderr.write("# %s capability check failed for uid %d with caps %s: expected '%s', got '%s'\n" %
-                        (proc_name, uid, captypes, expected, actual))
+        logger.info("%s capability check failed for uid %d with caps %s: expected '%s', got '%s'",
+                        proc_name, uid, captypes, expected, actual)
         return -1
     return 0
 
@@ -138,27 +137,27 @@ def test_some_caps_permitted():
 
 def test_some_caps_effective_non_root():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     return helper_test_some_caps(1000, ["effective", "permitted", "inheritable", "ambient"], 'CapEff')
 
 def test_some_caps_bounding_non_root():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     return helper_test_some_caps(1000, ["bounding"], 'CapBnd')
 
 def test_some_caps_inheritable_non_root():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     return helper_test_some_caps(1000, ["inheritable"], 'CapInh')
 
 def test_some_caps_ambient_non_root():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     return helper_test_some_caps(1000, ["ambient", "permitted", "inheritable"], 'CapAmb')
 
 def test_some_caps_permitted_non_root():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     return helper_test_some_caps(1000, ["ambient", "permitted", "inheritable"], 'CapPrm')
 
 
