@@ -28,6 +28,11 @@ def test_simple_delete():
     out, container_id = run_and_get_output(conf, detach=True, hide_stderr=True)
     if out != "":
         return -1
+
+    state = None
+    freezerCreated = False
+    cleanup_result = 0
+
     try:
         state = json.loads(run_crun_command(["state", container_id]))
         if state['status'] != "running":
@@ -35,35 +40,37 @@ def test_simple_delete():
         if state['id'] != container_id:
             return -1
     finally:
-        freezerCreated=False
-        if not os.path.exists("/sys/fs/cgroup/cgroup.controllers") and os.access('/sys/fs/cgroup/freezer/', os.W_OK):
-            # cgroupv1 freezer can easily simulate stuck or breaking `crun delete -f <cid>`
-            # this should be only done on cgroupv1 systems
-            if not os.path.exists("/sys/fs/cgroup/freezer/frozen/"):
-                freezerCreated=True
-                os.makedirs("/sys/fs/cgroup/freezer/frozen/")
-            with open('/sys/fs/cgroup/freezer/frozen/tasks', 'w') as f:
-                f.write(str(state['pid']))
-            with open('/sys/fs/cgroup/freezer/frozen/freezer.state', 'w') as f:
-                f.write('FROZEN')
-        try:
-            output = run_crun_command_raw(["delete", "-f", container_id])
-        except subprocess.CalledProcessError as exc:
-            print("Status : FAIL", exc.returncode, exc.output)
-            return -1
-        else:
-            # this is expected for cgroup v1 so ignore
-            if not output or b'Device or resource busy' in output:
-                # if output is empty or expected error pass
-                pass
+        if state is not None:
+            if not os.path.exists("/sys/fs/cgroup/cgroup.controllers") and os.access('/sys/fs/cgroup/freezer/', os.W_OK):
+                # cgroupv1 freezer can easily simulate stuck or breaking `crun delete -f <cid>`
+                # this should be only done on cgroupv1 systems
+                if not os.path.exists("/sys/fs/cgroup/freezer/frozen/"):
+                    freezerCreated = True
+                    os.makedirs("/sys/fs/cgroup/freezer/frozen/")
+                with open('/sys/fs/cgroup/freezer/frozen/tasks', 'w') as f:
+                    f.write(str(state['pid']))
+                with open('/sys/fs/cgroup/freezer/frozen/freezer.state', 'w') as f:
+                    f.write('FROZEN')
+            try:
+                output = run_crun_command_raw(["delete", "-f", container_id])
+            except subprocess.CalledProcessError as exc:
+                logger.error("Status : FAIL %s %s", exc.returncode, exc.output)
+                cleanup_result = -1
             else:
-                # anything else is error
-                print(output)
-                return -1
+                # this is expected for cgroup v1 so ignore
+                # also ignore profiling messages from coverage builds
+                if not output or b'Device or resource busy' in output or b'profiling:' in output:
+                    # if output is empty or expected error or profiling output, pass
+                    pass
+                else:
+                    # anything else is error
+                    logger.error(output)
+                    cleanup_result = -1
 
-        if freezerCreated:
-            os.rmdir("/sys/fs/cgroup/freezer/frozen/")
-    return 0
+            if freezerCreated:
+                os.rmdir("/sys/fs/cgroup/freezer/frozen/")
+
+    return cleanup_result
 
 def test_multiple_containers_delete():
     """Delete multiple containers with a regular expression"""
@@ -77,6 +84,9 @@ def test_multiple_containers_delete():
     out_test2, container_id_test2 = run_and_get_output(conf, detach=True, hide_stderr=True)
     if out_test2 != "":
         return -1
+
+    cleanup_result = 0
+
     try:
         state_test1 = json.loads(run_crun_command(["state", container_id_test1]))
         if state_test1['status'] != "running":
@@ -92,9 +102,10 @@ def test_multiple_containers_delete():
         try:
             output = run_crun_command_raw(["delete", "-f", "--regex", "test-*"])
         except subprocess.CalledProcessError as exc:
-            print("Status : FAIL", exc.returncode, exc.output)
-            return -1
-    return 0
+            logger.error("Status : FAIL %s %s", exc.returncode, exc.output)
+            cleanup_result = -1
+
+    return cleanup_result
 
 def test_help_delete():
     out = run_crun_command(["delete", "--help"])
