@@ -18,6 +18,10 @@
 import sys
 import copy
 import socket
+import os
+import shutil
+import subprocess
+import json
 from tests_utils import *
 import tempfile
 import re
@@ -52,14 +56,14 @@ def helper_mount(options: str, tmpfs: bool = True, userns: bool = False, is_file
             target = '/var/file' if is_file else '/var/dir'
             m = t.find_target(target)
             if m is None:
-                sys.stderr.write("# helper_mount failed: mount target '%s' not found in mountinfo\n" % target)
-                sys.stderr.write("# mount options: %s, tmpfs=%s, userns=%s, is_file=%s\n" % (options, tmpfs, userns, is_file))
-                sys.stderr.write("# mountinfo output: %s\n" % out[:300])
+                logger.info("helper_mount failed: mount target '%s' not found in mountinfo", target)
+                logger.info("mount options: %s, tmpfs=%s, userns=%s, is_file=%s", options, tmpfs, userns, is_file)
+                logger.info("mountinfo output: %s", out)
                 return [None, None]
             return [m.vfs_options, m.fs_options]
     except Exception as e:
-        sys.stderr.write("# helper_mount failed with exception: %s\n" % str(e))
-        sys.stderr.write("# mount options: %s, tmpfs=%s, userns=%s, is_file=%s\n" % (options, tmpfs, userns, is_file))
+        logger.info("helper_mount failed with exception: %s", e)
+        logger.info("mount options: %s, tmpfs=%s, userns=%s, is_file=%s", options, tmpfs, userns, is_file)
         return [None, None]
 
 def test_mount_symlink():
@@ -71,8 +75,8 @@ def test_mount_symlink():
     out, _ = run_and_get_output(conf, hide_stderr=True)
     if "Rome" in out:
         return 0
-    sys.stderr.write("# symlink mount test failed: expected 'Rome' in mountinfo output\n")
-    sys.stderr.write("# actual output: %s\n" % out[:200])
+    logger.info("symlink mount test failed: expected 'Rome' in mountinfo output")
+    logger.info("actual output: %s", out)
     return -1
 
 def test_mount_fifo():
@@ -89,8 +93,8 @@ def test_mount_fifo():
         conf['mounts'].append(mount_opt)
         out, _ = run_and_get_output(conf, hide_stderr=True)
         if "FIFO" not in out:
-            sys.stderr.write("# FIFO mount test failed with options %s: expected 'FIFO' in output\n" % options)
-            sys.stderr.write("# actual output: %s\n" % out)
+            logger.info("FIFO mount test failed with options %s: expected 'FIFO' in output", options)
+            logger.info("actual output: %s", out)
             return 1
     return 0
 
@@ -109,29 +113,32 @@ def test_mount_unix_socket():
         conf['mounts'].append(mount_opt)
         out, _ = run_and_get_output(conf, hide_stderr=True)
         if "socket" not in out:
-            sys.stderr.write("# unix socket mount test failed with options %s: expected 'socket' in output\n" % options)
-            sys.stderr.write("# actual output: %s\n" % out)
+            logger.info("unix socket mount test failed with options %s: expected 'socket' in output", options)
+            logger.info("actual output: %s", out)
             return 1
     return 0
 
 def test_mount_tmpfs_permissions():
     def prepare_rootfs(rootfs):
-        path = os.path.join(rootfs, "tmp")
+        path = os.path.join(rootfs, "test-tmpfs")
         os.mkdir(path)
         os.chmod(path, 0o712)
 
     conf = base_config()
-    conf['process']['args'] = ['/init', 'mode', '/tmp']
+    conf['process']['args'] = ['/init', 'mode', '/test-tmpfs']
     add_all_namespaces(conf)
-    conf['mounts'].append({"destination": "/tmp", "type": "tmpfs", "source": "tmpfs", "options": ["ro"]})
+    conf['mounts'].append({"destination": "/test-tmpfs", "type": "tmpfs", "source": "tmpfs", "options": ["ro"]})
     out, _ = run_and_get_output(conf, hide_stderr=True, callback_prepare_rootfs=prepare_rootfs)
     if "712" in out:
         return 0
-    sys.stderr.write("# tmpfs permissions test failed: expected '712' in mode output\n")
-    sys.stderr.write("# actual output: %s\n" % out)
+    logger.info("tmpfs permissions test failed: expected '712' in mode output")
+    logger.info("actual output: %s", out)
     return -1
 
 def test_mount_bind_to_rootfs():
+    if is_rootless():
+        return (77, "requires root for bind mount to rootfs")
+
     conf = base_config()
     conf['process']['args'] = ['/init', 'true']
     add_all_namespaces(conf)
@@ -202,7 +209,7 @@ def test_ro_cgroup():
                 for i in reversed(out.split("\n")):
                     if i.find("/sys/fs/cgroup") >= 0:
                         if i.find("ro,") < 0:
-                            print("fail with cgroupns=%s, netns=%s and cgroup_mount=%s, got %s" % (cgroupns, netns, has_cgroup_mount, i), file=sys.stderr)
+                            logger.error("fail with cgroupns=%s, netns=%s and cgroup_mount=%s, got %s", cgroupns, netns, has_cgroup_mount, i)
                             return -1
                         break
     return 0
@@ -442,7 +449,7 @@ def test_mount_dev():
 
 def test_userns_bind_mount():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     conf = base_config()
     add_all_namespaces(conf, userns=True)
 
@@ -473,7 +480,7 @@ def test_userns_bind_mount():
 
 def test_userns_bind_mount_symlink():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     conf = base_config()
     add_all_namespaces(conf, userns=True)
 
@@ -486,7 +493,7 @@ def test_userns_bind_mount_symlink():
     ]
     conf['linux']['uidMappings'] = fullMapping
     conf['linux']['gidMappings'] = fullMapping
-    sys.stderr.write("# start\n")
+    logger.info("start")
 
     bind_dir_parent = os.path.join(get_tests_root(), "bind-mount-userns-symlink")
     bind_dir = os.path.join(bind_dir_parent, "m")
@@ -502,17 +509,22 @@ def test_userns_bind_mount_symlink():
         os.chmod(bind_dir_parent, 0o000)
 
         conf['process']['args'] = ['/init', 'cat', "/foo/content"]
-        out, _ = run_and_get_output(conf, chown_rootfs_to=1)
+        out, _ = run_and_get_output(conf, chown_rootfs_to=1, hide_stderr=True)
         if out != "hello":
-            sys.stderr.write("# wrong file owner, found %s instead of %s\n" % (out, "hello"))
+            logger.info("wrong file content, found '%s' instead of 'hello'", out)
             return -1
     finally:
-        shutil.rmtree(bind_dir)
+        try:
+            # Restore permissions so we can clean up properly
+            os.chmod(bind_dir_parent, 0o755)
+            shutil.rmtree(bind_dir_parent)
+        except Exception as e:
+            logger.info("Failed to cleanup test directory: %s", e)
     return 0
 
 def test_idmapped_mounts():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     source_dir = os.path.join(get_tests_root(), "test-idmapped-mounts")
     try:
         os.makedirs(source_dir)
@@ -524,7 +536,7 @@ def test_idmapped_mounts():
 
         idmapped_mounts_status = subprocess.call([get_init_path(), "check-feature", "idmapped-mounts", source_dir])
         if idmapped_mounts_status != 0:
-            return 77
+            return (77, "idmapped mounts not supported")
 
         template = base_config()
         add_all_namespaces(template, userns=True)
@@ -557,7 +569,7 @@ def test_idmapped_mounts():
             conf['mounts'].append(mount_opt)
             out = run_and_get_output(conf, chown_rootfs_to=1)
             if expected not in out[0]:
-                sys.stderr.write("# wrong file owner, found %s instead of %s\n" % (out[0], expected))
+                logger.info("wrong file owner, found %s instead of %s", out[0], expected)
                 return True
             return False
 
@@ -641,13 +653,13 @@ def test_cgroup_mount_without_netns():
             if i.find("/sys/fs/cgroup") >= 0:
                 count = count + 1
         if count < 2:
-            sys.stderr.write("# fail with cgroupns=%s, got %s\n" % (cgroupns, out))
+            logger.info("fail with cgroupns=%s, got %s", cgroupns, i)
             return -1
     return 0
 
 def test_add_remove_mounts():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     conf = base_config()
 
     conf['mounts'].append({"destination": "/foo", "type": "tmpfs", "source": "tmpfs", "options": ["rw"]})
@@ -672,9 +684,9 @@ def test_add_remove_mounts():
         if exists == expected:
             return True
         if expected:
-            sys.stderr.write("# test file not found\n")
+            logger.info("test file not found")
         else:
-            sys.stderr.write("# test file found\n")
+            logger.info("test file found")
         return False
 
     new_mounts = [{"destination": parent_dir_in_container, "type": "bind", "source": bind_dir, "options": ["bind", "ro"]},
@@ -695,7 +707,7 @@ def test_add_remove_mounts():
             return -1
         out = run_crun_command(["exec", cid, "/init", "cat", "/proc/self/mountinfo"])
         if not re.search(r".*/ /foo/tmpfs .*tmpfs.*", out):
-            sys.stderr.write("# /foo/tmpfs not found as a tmpfs\n")
+            logger.info("/foo/tmpfs not found as a tmpfs")
             return -1
 
         run_crun_command(["mounts", "remove", cid, mounts_path])
@@ -704,7 +716,7 @@ def test_add_remove_mounts():
 
         out = run_crun_command(["exec", cid, "/init", "cat", "/proc/self/mountinfo"])
         if re.search(r".*/ /foo/tmpfs .*tmpfs.*", out):
-            sys.stderr.write("# /foo/tmpfs still mounted\n")
+            logger.info("/foo/tmpfs still mounted")
             return -1
     finally:
         if cid is not None:
@@ -764,12 +776,12 @@ def test_bind_mount_symlink_nofollow():
             conf['mounts'].append(mount_opt)
 
             try:
-                out, _ = run_and_get_output(conf, hide_stderr=True,callback_prepare_rootfs=prepare_rootfs)
-                sys.stderr.write("# got output %s with configuration userns=%s, src-nofollow=%s\n" % (out, userns, src_nofollow))
+                out, _ = run_and_get_output(conf, hide_stderr=True, callback_prepare_rootfs=prepare_rootfs)
+                logger.info("got output %s with configuration userns=%s, src-nofollow=%s", out, userns, src_nofollow)
                 if expected not in out:
                     return -1
             except Exception as e:
-                sys.stderr.write("# error %s\n" % e)
+                logger.info("error %s", e)
                 return -1
 
     return 0
@@ -789,10 +801,10 @@ def test_bind_mount_symlink_nofollow_procfs():
     conf['mounts'].append(mount_opt)
 
     try:
-        out, _ = run_and_get_output(conf, hide_stderr=True,callback_prepare_rootfs=prepare_rootfs)
+        out, _ = run_and_get_output(conf, hide_stderr=True, callback_prepare_rootfs=prepare_rootfs)
         return -1
     except Exception as e:
-        sys.stderr.write("# error %s\n" % e)
+        logger.info("error %s", e)
         return 0
 
     return 0
@@ -834,17 +846,17 @@ def test_bind_mount_file_nofollow():
             conf['mounts'].append(mount_opt)
 
             try:
-                out, _ = run_and_get_output(conf, hide_stderr=True,callback_prepare_rootfs=prepare_rootfs)
-                sys.stderr.write("# got output %s with configuration userns=%s, src-nofollow=%s\n" % (out, userns, src_nofollow))
+                out, _ = run_and_get_output(conf, hide_stderr=True, callback_prepare_rootfs=prepare_rootfs)
+                logger.info("got output %s with configuration userns=%s, src-nofollow=%s", out, userns, src_nofollow)
                 if target_content not in out:
                     return 1
             except Exception as e:
-                sys.stderr.write("# error %s\n" % e)
+                logger.info("error %s", e)
     return 0
 
 def test_idmapped_mounts_without_userns():
     if is_rootless():
-        return 77
+        return (77, "requires root privileges")
     source_dir = os.path.join(get_tests_root(), "test-idmapped-mounts-no-userns")
     try:
         os.makedirs(source_dir)
@@ -875,14 +887,71 @@ def test_idmapped_mounts_without_userns():
         out, _ = run_and_get_output(conf, hide_stderr=True)
 
         if "1000:1000" not in out:
-            sys.stderr.write("# idmap without userns test failed: expected '1000:1000' in output\n")
-            sys.stderr.write("# actual output: %s\n" % out)
+            logger.info("idmap without userns test failed: expected '1000:1000' in output")
+            logger.info("actual output: %s", out)
             return 1
     finally:
         shutil.rmtree(source_dir)
 
     return 0
 
+
+def test_annotation_mount_context_type():
+    """Test run.oci.mount_context_type annotation for SELinux mount contexts."""
+    # Check if SELinux is available and enabled
+    try:
+        with open('/sys/fs/selinux/enforce', 'r') as f:
+            selinux_enabled = f.read().strip() in ['0', '1']
+    except Exception:
+        return (77, "SELinux not available")
+
+    if not selinux_enabled:
+        return (77, "SELinux not enabled")
+
+    conf = base_config()
+    add_all_namespaces(conf)
+    conf['process']['args'] = ['/init', 'cat', '/proc/self/mountinfo']
+
+    # Create a tmpfs mount to test SELinux context
+    mount_opt = {
+        "destination": "/test-selinux",
+        "type": "tmpfs",
+        "source": "tmpfs",
+        "options": ["rw"]
+    }
+    conf['mounts'].append(mount_opt)
+
+    # Test different context types
+    for context_type in ['context', 'fscontext', 'defcontext', 'rootcontext']:
+        logger.info("testing mount_context_type: %s", context_type)
+
+        # Add annotation for mount context type
+        if 'annotations' not in conf:
+            conf['annotations'] = {}
+        conf['annotations']['run.oci.mount_context_type'] = context_type
+
+        try:
+            out, _ = run_and_get_output(conf, hide_stderr=True)
+            logger.info("mount_context_type=%s test passed", context_type)
+            # Just verify it doesn't crash - actual SELinux context verification
+            # would require checking specific SELinux contexts which vary by system
+
+        except subprocess.CalledProcessError as e:
+            output = e.output.decode('utf-8', errors='ignore') if e.output else ''
+            if any(x in output.lower() for x in ["mount", "proc", "permission", "rootfs", "private", "busy"]):
+                return (77, "not available in nested namespaces")
+            if "selinux" in output.lower() or "context" in output.lower():
+                # SELinux context issues are acceptable - may not be fully configured
+                logger.info("mount_context_type=%s skipped due to SELinux configuration", context_type)
+                continue
+            logger.info("test failed for context_type=%s: %s", context_type, e)
+            return -1
+        except Exception as e:
+            logger.info("test failed for context_type=%s: %s", context_type, e)
+            return -1
+
+    return 0
+
 all_tests = {
     "mount-ro" : test_mount_ro,
     "mount-rro" : test_mount_rro,
@@ -919,6 +988,7 @@ all_tests = {
     "mount-tmpfs-permissions": test_mount_tmpfs_permissions,
     "mount-add-remove-mounts": test_add_remove_mounts,
     "mount-help": test_mount_help,
+    "annotation-mount-context-type": test_annotation_mount_context_type,
 }
 
 if __name__ == "__main__":
