@@ -16,6 +16,7 @@
 # along with crun.  If not, see <http://www.gnu.org/licenses/>.
 
 import json
+import logging
 import shutil
 import sys
 import os
@@ -26,6 +27,21 @@ import time
 
 default_umask = 0o22
 
+# Simple logging setup for TAP diagnostics
+logging.basicConfig(
+    level=logging.INFO,
+    format='# %(message)s',
+    stream=sys.stderr
+)
+logger = logging.getLogger('crun.tests')
+
+# Export logger for use in test files
+__all__ = ['logger', 'base_config', 'run_and_get_output', 'run_crun_command', 'run_crun_command_raw',
+           'parse_proc_status', 'add_all_namespaces', 'tests_main', 'is_rootless',
+           'is_cgroup_v2_unified', 'get_crun_feature_string', 'running_on_systemd',
+           'get_tests_root', 'get_tests_root_status', 'get_init_path', 'get_crun_path',
+           'get_cgroup_manager', 'get_test_environment']
+
 base_conf = """
 {
     "ociVersion": "1.0.0",
@@ -122,6 +138,17 @@ base_conf = """
 		"noexec",
 		"nodev"
 	    ]
+	},
+	{
+	    "destination": "/tmp",
+	    "type": "tmpfs",
+	    "source": "tmpfs",
+	    "options": [
+		"nosuid",
+		"nodev",
+		"mode=1777",
+		"size=65536k"
+	    ]
 	}
     ],
     "linux": {
@@ -176,6 +203,10 @@ def run_all_tests(all_tests, allowed_tests):
         allowed_tests = allowed_tests.split()
         tests = {k: v for k, v in tests.items() if k in allowed_tests}
 
+    # Test timing thresholds
+    SLOW_TEST_THRESHOLD = 30.0  # seconds
+    VERY_SLOW_TEST_THRESHOLD = 60.0  # seconds
+
     print("1..%d" % len(tests))
     cur = 0
     for k, v in tests.items():
@@ -189,22 +220,67 @@ def run_all_tests(all_tests, allowed_tests):
 
             ret = v()
             test_duration = time.time() - test_start_time
+
+            # Check for slow tests and emit warnings
+            if test_duration > VERY_SLOW_TEST_THRESHOLD:
+                logger.warning("Test '%s' took %.3fs (>%.1fs very slow threshold)",
+                              k, test_duration, VERY_SLOW_TEST_THRESHOLD)
+            elif test_duration > SLOW_TEST_THRESHOLD:
+                logger.warning("Test '%s' took %.3fs (>%.1fs slow threshold)",
+                              k, test_duration, SLOW_TEST_THRESHOLD)
+
             if ret == 0:
                 print("ok %d - %s # %.3fs" % (cur, k, test_duration))
-            elif ret == 77:
-                print("ok %d - %s #SKIP # %.3fs" % (cur, k, test_duration))
+            elif ret == 77 or (isinstance(ret, tuple) and ret[0] == 77):
+                skip_reason = ""
+                if isinstance(ret, tuple) and len(ret) > 1 and ret[1]:
+                    skip_reason = " " + str(ret[1])
+                print("ok %d - %s #SKIP%s # %.3fs" % (cur, k, skip_reason, test_duration))
             else:
+                actual_ret = ret[0] if isinstance(ret, tuple) else ret
                 print("not ok %d - %s # %.3fs" % (cur, k, test_duration))
-                sys.stderr.write("# Test '%s' failed with return code %d\n" % (k, ret))
+                logger.error("Test '%s' failed with return code %d", k, actual_ret)
+                # Log environment context for failed tests
+                try:
+                    env_info = get_test_environment()
+                    logger.error("Test environment: uid=%d rootless=%s cgroup_v2=%s cgroup_manager=%s",
+                                env_info['uid'], env_info['rootless'], env_info['cgroup_v2'], env_info['cgroup_manager'])
+                except Exception:
+                    pass
         except Exception as e:
             test_duration = time.time() - test_start_time
-            sys.stderr.write("# Test '%s' failed with exception after %.3fs:\n" % (k, test_duration))
+            logger.error("Test '%s' failed with exception after %.3fs:", k, test_duration)
+            logger.error("Exception type: %s", type(e).__name__)
+            logger.error("Exception message: %s", str(e))
+
+            # Enhanced error details for subprocess errors
+            if hasattr(e, 'returncode'):
+                logger.error("Process return code: %d", e.returncode)
+            if hasattr(e, 'cmd'):
+                cmd_str = ' '.join(e.cmd) if isinstance(e.cmd, list) else str(e.cmd)
+                logger.error("Failed command: %s", cmd_str)
             if hasattr(e, 'output'):
-                sys.stderr.write("# Container output: %s\n" % str(e.output))
-            sys.stderr.write("# Exception: %s\n" % str(e))
-            sys.stderr.write("# Traceback:\n")
+                logger.error("Process output: %s", str(e.output))
+            if hasattr(e, 'stderr') and e.stderr:
+                logger.error("Process stderr: %s", str(e.stderr))
+
+            # Environment information
+            logger.error("Working directory: %s", os.getcwd())
+            logger.error("Test root: %s", get_tests_root())
+            if 'TMPDIR' in os.environ:
+                logger.error("TMPDIR: %s", os.environ['TMPDIR'])
+
+            # Test environment context
+            try:
+                env_info = get_test_environment()
+                logger.error("Test environment: uid=%d rootless=%s cgroup_v2=%s cgroup_manager=%s",
+                            env_info['uid'], env_info['rootless'], env_info['cgroup_v2'], env_info['cgroup_manager'])
+            except Exception:
+                pass
+
+            logger.error("Traceback:")
             for line in traceback.format_exc().splitlines():
-                sys.stderr.write("# %s\n" % line)
+                logger.error("%s", line)
             ret = -1
             print("not ok %d - %s # %.3fs" % (cur, k, test_duration))
 
@@ -221,9 +297,37 @@ def get_crun_path():
     cwd = os.getcwd()
     return os.getenv("OCI_RUNTIME") or os.path.join(cwd, "crun")
 
+def get_cgroup_manager():
+    """Get cgroup manager from CGROUP_MANAGER env var or default to cgroupfs."""
+    return os.getenv('CGROUP_MANAGER', 'cgroupfs')
+
+def get_test_environment():
+    """Return a dict describing the current test environment.
+
+    Useful for debugging which code paths are exercised in different environments.
+    """
+    in_userns = False
+    try:
+        with open('/proc/self/uid_map') as f:
+            content = f.read()
+            # If we have a full mapping (4294967295), we're not in a restricted userns
+            in_userns = '4294967295' not in content
+    except:
+        pass
+
+    return {
+        'uid': os.getuid(),
+        'gid': os.getgid(),
+        'rootless': is_rootless(),
+        'systemd': running_on_systemd(),
+        'cgroup_v2': is_cgroup_v2_unified(),
+        'cgroup_manager': get_cgroup_manager(),
+        'in_userns': in_userns,
+    }
+
 def run_and_get_output(config, detach=False, preserve_fds=None, pid_file=None,
                        keep=False,
-                       command='run', env=None, use_popen=False, hide_stderr=False, cgroup_manager='cgroupfs',
+                       command='run', env=None, use_popen=False, hide_stderr=False, cgroup_manager=None,
                        all_dev_null=False, stdin_dev_null=False, id_container=None, relative_config_path="config.json",
                        chown_rootfs_to=None, callback_prepare_rootfs=None, debug=False):
 
@@ -287,6 +391,10 @@ def run_and_get_output(config, detach=False, preserve_fds=None, pid_file=None,
     relative_config_path = ['--config', relative_config_path] if relative_config_path else []
     debug_arg = ['--debug'] if debug else []
 
+    # Use env var if cgroup_manager not explicitly specified
+    if cgroup_manager is None:
+        cgroup_manager = get_cgroup_manager()
+
     root = get_tests_root_status()
     args = [crun] + debug_arg + ["--cgroup-manager", cgroup_manager, "--root", root, command] + relative_config_path + preserve_fds_arg + detach_arg + keep_arg + pid_file_arg + [id_container]
 
@@ -316,15 +424,15 @@ def run_and_get_output(config, detach=False, preserve_fds=None, pid_file=None,
         try:
             return subprocess.check_output(args, cwd=temp_dir, stdin=stdin, stderr=stderr, env=env, close_fds=False, umask=default_umask).decode(), id_container
         except subprocess.CalledProcessError as e:
-            sys.stderr.write("# Command failed: %s\n" % ' '.join(args))
-            sys.stderr.write("# Working directory: %s\n" % temp_dir)
-            sys.stderr.write("# Container ID: %s\n" % id_container)
-            sys.stderr.write("# Return code: %d\n" % e.returncode)
+            logger.error("Command failed: %s", ' '.join(args))
+            logger.error("Working directory: %s", temp_dir)
+            logger.error("Container ID: %s", id_container)
+            logger.error("Return code: %d", e.returncode)
             if e.output:
-                sys.stderr.write("# Output: %s\n" % e.output.decode('utf-8', errors='ignore'))
-            sys.stderr.write("# Config file saved at: %s\n" % config_path)
+                logger.error("Output: %s", e.output.decode('utf-8', errors='ignore'))
+            logger.error("Config file saved at: %s", config_path)
             if not keep:
-                sys.stderr.write("# Note: temporary directory will be cleaned up\n")
+                logger.error("Note: temporary directory will be cleaned up")
             raise
 
 def run_crun_command(args):
@@ -334,10 +442,10 @@ def run_crun_command(args):
     try:
         return subprocess.check_output(cmd_args, close_fds=False).decode()
     except subprocess.CalledProcessError as e:
-        sys.stderr.write("# crun command failed: %s\n" % ' '.join(cmd_args))
-        sys.stderr.write("# Return code: %d\n" % e.returncode)
+        logger.error("crun command failed: %s", ' '.join(cmd_args))
+        logger.error("Return code: %d", e.returncode)
         if e.output:
-            sys.stderr.write("# Output: %s\n" % e.output.decode('utf-8', errors='ignore'))
+            logger.error("Output: %s", e.output.decode('utf-8', errors='ignore'))
         raise
 
 # Similar as run_crun_command but does not performs decode of output and relays error message for further matching
@@ -348,8 +456,8 @@ def run_crun_command_raw(args):
     try:
         return subprocess.check_output(cmd_args, close_fds=False, stderr=subprocess.STDOUT)
     except subprocess.CalledProcessError as e:
-        sys.stderr.write("# crun command failed: %s\n" % ' '.join(cmd_args))
-        sys.stderr.write("# Return code: %d\n" % e.returncode)
+        logger.error("crun command failed: %s", ' '.join(cmd_args))
+        logger.error("Return code: %d", e.returncode)
         raise
 
 def running_on_systemd():
